import re
import subprocess

from pyhap.accessory import Accessory
from pyhap.const import CATEGORY_SWITCH

PLIST_RE = re.compile(r'^"(?P<key>.*?)" = <?"?(?P<value>.*?)"?>?$')


def get_display_state():
    result = subprocess.check_output(['pmset', '-g', 'powerstate', 'IODisplayWrangler'])
    return int(result.strip().split(b'\n')[-1].split()[1]) >= 4


def set_display_state(state):
    if state:
        subprocess.call(['caffeinate', '-u', '-t', '1'])
    else:
        subprocess.call(['pmset', 'displaysleepnow'])


def get_machine_info():
    result = [
        x.strip().decode()
        for x in subprocess.check_output(['ioreg', '-c', 'IOPlatformExpertDevice', '-d', '2']).split(b'\n')
    ]
    result = result[result.index('{') + 1:result.index('}')]

    data = {
        key: value
        for key, value in [
            PLIST_RE.match(row).groups()
            for row in result
            if PLIST_RE.match(row)
        ]
    }

    return {
        'SerialNumber': data['IOPlatformSerialNumber'],
        'Model': data['model'],
        'Manufacturer': data['manufacturer'],
    }


class DisplaySwitch(Accessory):
    category = CATEGORY_SWITCH

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.display = self.add_preload_service('Switch').configure_char(
            'On', setter_callback=self.set_display
        )

    def add_info_service(self):
        info_service = self.driver.loader.get_service('AccessoryInformation')
        info_service.configure_char('Name', value=self.display_name)
        for name, value in get_machine_info().items():
            info_service.configure_char(name, value=value)
        self.add_service(info_service)

    @Accessory.run_at_interval(1)
    def run(self):
        state = get_display_state()

        if self.display.value != state:
            self.display.value = state
            self.display.notify()

    def set_display(self, state):
        if get_display_state() != state:
            set_display_state(state)



import inspect
from pathlib import Path
import subprocess

PLIST = '''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
 <key>KeepAlive</key>
 <true/>
 <key>Label</key>
 <string>net.schinckel.homekit-display-switch</string>
 <key>ProgramArguments</key>
 <array>
  <string>{path}</string>
 </array>
 <key>RunAtLoad</key>
 <true/>
</dict>
</plist>'''  # NOQA

target_file = Path('~/Library/LaunchAgents/net.schinckel.homekit-display-switch.plist').expanduser()


def main():
    # We need to see if we can look at the first item on the stack to see where we were called
    # from, and see if our matching 'homekit-display-switch' command is in there, because
    # we want to use that as the path.
    top = inspect.stack()[-1]
    path = Path(top.filename).parent / 'homekit-display-switch'
    if path.exists():
        data = PLIST.format(path=path.resolve())
        target_file.write_text(data)
        subprocess.call([
            'launchctl', 'load', '-w', target_file
        ])
    else:
        raise Exception('Unable to find "homekit-display-switch".')



from setuptools import setup


setup(
    name='HomeKit-Display-Switch',
    version='1.0.1',
    author='Matthew Schinckel',
    author_email='matt@schinckel.net',
    description="HomeKit accessory that allows you to sleep/wake your Mac's display.",
    long_description=open('README.rst').read(),
    packages=['display_switch'],
    url='https://bitbucket.org/schinckel/homekit-display-switch',
    entry_points={
        'console_scripts': [
            'homekit-display-switch = display_switch.__main__:main',
            'load-homekit-display-switch = display_switch.load_service:main',
        ]
    },
    install_requires=[
        'HAP-python[QRCode]',
        'base36',
        'pyqrcode',
    ],
    python_requires=">3.6",
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: MacOS X',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Home Automation',
    ]
)



import random
import signal
import subprocess
from pathlib import Path

from pyhap.accessory_driver import AccessoryDriver

from .display_switch import DisplaySwitch

path = Path(__file__).parent / 'display-switch.state'


def main():
    driver = AccessoryDriver(
        port=random.randint(50000, 60000),
        persist_file=path.resolve(),
    )
    driver.add_accessory(accessory=DisplaySwitch(
        driver,
        subprocess.check_output(['hostname', '-s']).strip().decode(),
    ))

    signal.signal(signal.SIGTERM, driver.signal_handler)
    driver.start()
